@@ -28,15 +28,17 @@ Methods
 
 """
 
-from typing import Optional, Dict, Any
+from io import BytesIO
+from typing import Callable, Optional, Dict, Any, Tuple, cast, Iterator, Union, List
 import numpy as np
-from PIL import Image, UnidentifiedImageError, ImageSequence, ExifTags
+from PIL import Image, UnidentifiedImageError, ImageSequence, ExifTags  # type: ignore
 from ..core.request import Request, IOMode, InitializationError, URI_BYTES
 from ..core.v3_plugin_api import PluginV3, ImageProperties
 import warnings
+from ..typing import ArrayLike
 
 
-def _exif_orientation_transform(orientation, mode):
+def _exif_orientation_transform(orientation: int, mode: str) -> Callable:
     # get transformation that transforms an image from a
     # given EXIF orientation into the standard orientation
 
@@ -70,7 +72,7 @@ class PillowPlugin(PluginV3):
 
         super().__init__(request)
 
-        self._image = None
+        self._image: Image = None
 
         if request.mode.io_mode == IOMode.read:
             try:
@@ -118,7 +120,7 @@ class PillowPlugin(PluginV3):
         self._request.finish()
 
     def read(
-        self, *, index=0, mode=None, rotate=False, apply_gamma=False
+        self, *, index=None, mode=None, rotate=False, apply_gamma=False
     ) -> np.ndarray:
         """
         Parses the given URI and creates a ndarray from it.
@@ -126,10 +128,13 @@ class PillowPlugin(PluginV3):
         Parameters
         ----------
         index : {integer}
-            If the URI contains a list of ndimages (multiple frames) return the
-            index-th image/frame. If None, read all ndimages (frames) in the URI
-            and attempt to stack them along a new 0-th axis (equivalent to
-            np.stack(imgs, axis=0))
+            If the ImageResource contains multiple ndimages, and index is an
+            integer, select the index-th ndimage from among them and return it.
+            If index is an ellipsis (...), read all ndimages in the file and
+            stack them along a new batch dimension and return them. If index is
+            None, this plugin reads the first image of the file (index=0) unless
+            the image is a GIF or APNG, in which case all images are read
+            (index=...).
         mode : {str, None}
             Convert the image to the given mode before returning it. If None,
             the mode will be left unchanged. Possible modes can be found at:
@@ -155,7 +160,15 @@ class PillowPlugin(PluginV3):
 
         """
 
-        if index is not None:
+        if index is None:
+            if self._image.format == "GIF":
+                index = Ellipsis
+            elif self._image.custom_mimetype == "image/apng":
+                index = Ellipsis
+            else:
+                index = 0
+
+        if isinstance(index, int):
             # will raise IO error if index >= number of frames in image
             self._image.seek(index)
             image = self._apply_transforms(self._image, mode, rotate, apply_gamma)
@@ -165,7 +178,9 @@ class PillowPlugin(PluginV3):
             image = np.stack([im for im in iterator], axis=0)
             return image
 
-    def iter(self, *, mode=None, rotate=False, apply_gamma=False) -> np.ndarray:
+    def iter(
+        self, *, mode: str = None, rotate: bool = False, apply_gamma: bool = False
+    ) -> Iterator[np.ndarray]:
         """
         Iterate over all ndimages/frames in the URI
 
@@ -212,7 +227,12 @@ class PillowPlugin(PluginV3):
         return image
 
     def write(
-        self, image: np.ndarray, *, mode=None, format=None, **kwargs
+        self,
+        ndimage: Union[ArrayLike, List[ArrayLike]],
+        *,
+        mode: str = None,
+        format: str = None,
+        **kwargs,
     ) -> Optional[bytes]:
         """
         Write an ndimage to the URI specified in path.
@@ -259,15 +279,24 @@ class PillowPlugin(PluginV3):
             "format": format or Image.registered_extensions()[extension],
         }
 
+        if isinstance(ndimage, list):
+            ndimage = np.stack(ndimage, axis=0)
+            is_batch = True
+        else:
+            ndimage = np.asarray(ndimage)
+            is_batch = None
+
         # check if ndimage is a batch of frames/pages (e.g. for writing GIF)
         # if mode is given, use it; otherwise fall back to image.ndim only
+        if is_batch is True:
+            pass  # ndimage was list; we know it is a batch
         if mode is not None:
             is_batch = (
-                image.ndim > 3 if Image.getmodebands(mode) > 1 else image.ndim > 2
+                ndimage.ndim > 3 if Image.getmodebands(mode) > 1 else ndimage.ndim > 2
             )
-        elif image.ndim == 2:
+        elif ndimage.ndim == 2:
             is_batch = False
-        elif image.ndim == 3 and image.shape[-1] in [2, 3, 4]:
+        elif ndimage.ndim == 3 and ndimage.shape[-1] in [2, 3, 4]:
             # Note: this makes a channel-last assumption
             # (pillow seems to make it as well)
             is_batch = False
@@ -275,10 +304,10 @@ class PillowPlugin(PluginV3):
             is_batch = True
 
         if not is_batch:
-            image = image[None, ...]
+            ndimage = ndimage[None, ...]
 
         pil_frames = list()
-        for frame in image:
+        for frame in ndimage:
             pil_frame = Image.fromarray(frame, mode=mode)
             if "bits" in kwargs:
                 pil_frame = pil_frame.quantize(colors=2 ** kwargs["bits"])
@@ -293,23 +322,45 @@ class PillowPlugin(PluginV3):
         primary_image.save(self._request.get_file(), **save_args)
 
         if self._request._uri_type == URI_BYTES:
-            return self._request.get_file().getvalue()
+            file = cast(BytesIO, self._request.get_file())
+            return file.getvalue()
+
+        return None
 
     def get_meta(self, *, index=0) -> Dict[str, Any]:
         return self.metadata(index=index, exclude_applied=False)
 
-    def metadata(self, index: int = 0, exclude_applied: bool = True) -> Dict[str, Any]:
-        """Read ndimage metadata from the URI
+    def metadata(
+        self, index: int = None, exclude_applied: bool = True
+    ) -> Dict[str, Any]:
+        """Read ndimage metadata.
 
         Parameters
         ----------
         index : {integer, None}
-            If the URI contains a list of ndimages return the metadata
-            corresponding to the index-th image. If None, return the metadata
-            for the last read ndimage/frame.
+            If the ImageResource contains multiple ndimages, and index is an
+            integer, select the index-th ndimage from among them and return its
+            metadata. If index is an ellipsis (...), read and return global
+            metadata. If index is None, this plugin reads metadata from the
+            first image of the file (index=0) unless the image is a GIF or APNG,
+            in which case global metadata is read (index=...).
+
+        Returns
+        -------
+        metadata : dict
+            A dictionary of format-specific metadata.
+
         """
 
-        if index is not None and self._image.tell() != index:
+        if index is None:
+            if self._image.format == "GIF":
+                index = Ellipsis
+            elif self._image.custom_mimetype == "image/apng":
+                index = Ellipsis
+            else:
+                index = 0
+
+        if isinstance(index, int) and self._image.tell() != index:
             self._image.seek(index)
 
         metadata = self._image.info.copy()
@@ -332,24 +383,40 @@ class PillowPlugin(PluginV3):
 
         return metadata
 
-    def properties(self, index: int = 0) -> ImageProperties:
+    def properties(self, index: int = None) -> ImageProperties:
         """Standardized ndimage metadata
         Parameters
         ----------
         index : int
-            The index of the ndimage for which to return properties. If the
-            index is out of bounds a ``ValueError`` is raised. If ``None``,
-            return the properties for the ndimage stack. If this is impossible,
-            e.g., due to shape missmatch, an exception will be raised.
+            If the ImageResource contains multiple ndimages, and index is an
+            integer, select the index-th ndimage from among them and return its
+            properties. If index is an ellipsis (...), read and return the
+            properties of all ndimages in the file stacked along a new batch
+            dimension. If index is None, this plugin reads and returns the
+            properties of the first image (index=0) unless the image is a GIF or
+            APNG, in which case it reads and returns the properties all images
+            (index=...).
 
         Returns
         -------
         properties : ImageProperties
             A dataclass filled with standardized image metadata.
 
+        Notes
+        -----
+        This does not decode pixel data and is 394fast for large images.
+
         """
 
         if index is None:
+            if self._image.format == "GIF":
+                index = Ellipsis
+            elif self._image.custom_mimetype == "image/apng":
+                index = Ellipsis
+            else:
+                index = 0
+
+        if index is Ellipsis:
             self._image.seek(0)
         else:
             self._image.seek(index)
@@ -360,14 +427,21 @@ class PillowPlugin(PluginV3):
         else:
             mode = self._image.mode
 
-        dummy = np.asarray(Image.new(mode, (1, 1)))
+        width: int = self._image.width
+        height: int = self._image.height
+        shape: Tuple[int, ...] = (height, width)
 
-        shape = list(dummy.shape)
-        shape[:2] = self._image.size[::-1]
-        shape = (self._image.n_frames, *shape) if index is None else tuple(shape)
+        n_frames: int = self._image.n_frames
+        if index is ...:
+            shape = (n_frames, *shape)
+
+        dummy = np.asarray(Image.new(mode, (1, 1)))
+        pil_shape: Tuple[int, ...] = dummy.shape
+        if len(pil_shape) > 2:
+            shape = (*shape, *pil_shape[2:])
 
         return ImageProperties(
             shape=shape,
             dtype=dummy.dtype,
-            is_batch=True if index is None else False,
+            is_batch=True if index is Ellipsis else False,
         )
