@@ -8,7 +8,7 @@ import os
 import io
 import pytest
 import numpy as np
-from PIL import Image, ImageSequence
+from PIL import Image, ImageSequence  # type: ignore
 
 import imageio as iio
 from imageio.core.v3_plugin_api import PluginV3
@@ -186,7 +186,7 @@ def test_jpg_compression(test_images, tmp_path):
 
 
 def test_exif_orientation(test_images, tmp_path):
-    from PIL.Image import Exif
+    from PIL.Image import Exif  # type: ignore
 
     im = np.load(test_images / "chelsea.npy")
 
@@ -204,7 +204,7 @@ def test_exif_orientation(test_images, tmp_path):
         exif=exif_tag,
     )
 
-    with iio.imopen(
+    with iio.v3.imopen(
         tmp_path / "chelsea_tagged.png",
         "r",
         plugin="pillow",
@@ -265,7 +265,7 @@ def test_gif_irregular_duration(test_images, tmp_path):
     )
     duration = [0.5 if idx in [2, 5, 7] else 0.1 for idx in range(im.shape[0])]
 
-    with iio.imopen(tmp_path / "test.gif", "w", plugin="pillow") as file:
+    with iio.v3.imopen(tmp_path / "test.gif", "w", plugin="pillow") as file:
         for frame, duration in zip(im, duration):
             file.write(frame, duration=duration)
 
@@ -293,7 +293,7 @@ def test_gif_loop_and_fps(test_images, tmp_path):
         mode="RGBA",
     )
 
-    with iio.imopen(tmp_path / "test.gif", "w", plugin="pillow") as file:
+    with iio.v3.imopen(tmp_path / "test.gif", "w", plugin="pillow") as file:
         for frame in im:
             file.write(frame, palettesize=100, fps=20, loop=2)
 
@@ -304,7 +304,7 @@ def test_gif_indexed_read(test_images):
     idx = 0
     numpy_im = np.load(test_images / "newtonscradle_rgb.npy")[idx, ...]
 
-    with iio.imopen(test_images / "newtonscradle.gif", "r", plugin="pillow") as file:
+    with iio.v3.imopen(test_images / "newtonscradle.gif", "r", plugin="pillow") as file:
         # exists to touch branch, would be better two write an explicit test
         meta = file.get_meta(index=idx)
         assert "version" in meta
@@ -323,47 +323,21 @@ def test_unknown_image(tmp_path):
         PillowPlugin(r)
 
 
-# TODO: introduce new plugin for writing compressed GIF
-# This is not what pillow does, and hence unexpected when explicitly calling
-# for pillow
-# def test_gif_subrectangles(test_images, tmp_path):
-#     # feature might be made obsolete by upstream (pillow) supporting it natively
-#     # related issues: https://github.com/python-pillow/Pillow/issues/4977
-#     im = iio.v3.imread(test_images / "newtonscradle.gif", legacy_api=False, plugin="pillow", mode="RGBA")
-#     im = np.stack((*im, im[-1]), axis=0)
-#     print(im.dtype)
-
-#     with iio.imopen(tmp_path / "1.gif", legacy_api=False, plugin="pillow") as f:
-#         f.write(im, subrectangles=False, mode="RGBA")
-
-#     with iio.imopen(tmp_path / "2.gif", legacy_api=False, plugin="pillow") as f:
-#         f.write(im, subrectangles=True, mode="RGBA")
-
-#     size_1 = os.stat(tmp_path / "1.gif").st_size
-#     size_2 = os.stat(tmp_path / "2.gif").st_size
-#     assert size_2 < size_1
-
-
 def test_gif_transparent_pixel(test_images):
     # see issue #245
     im = iio.v3.imread(
-        test_images / "imageio_issue245.gif",
-        plugin="pillow",
-        mode="RGBA",
+        test_images / "imageio_issue245.gif", plugin="pillow", mode="RGBA", index=0
     )
     assert im.shape == (24, 30, 4)
 
 
-# TODO: Pillow actually doesn't read zip. This should be a different plugin.
-# def test_inside_zipfile(test_images):
+def test_gif_list_write(test_images, tmp_path):
+    im = iio.v3.imread(test_images / "imageio_issue245.gif", plugin="pillow")
+    im_list = [x for x in im]
+    iio.v3.imwrite(tmp_path / "test.gif", im_list, plugin="pillow")
+    im2 = iio.v3.imread(test_images / "imageio_issue245.gif", plugin="pillow", index=0)
 
-#     fname = os.path.join(tmp_path, "pillowtest.zip")
-#     with ZipFile(fname, "w") as z:
-#         z.writestr("x.png", open(test_images / "chelsea.png", "rb").read())
-#         z.writestr("x.jpg", open(test_images / "rommel.jpg", "rb").read())
-
-#     for name in ("x.png", "x.jpg"):
-#         imageio.imread(fname + "/" + name)
+    assert im2.shape == (24, 30, 3)
 
 
 def test_legacy_exif_orientation(test_images, tmp_path):
@@ -385,7 +359,7 @@ def test_legacy_exif_orientation(test_images, tmp_path):
         exif=exif_tag,
     )
 
-    with iio.imopen(
+    with iio.v3.imopen(
         tmp_path / "chelsea_tagged.png",
         "r",
         legacy_mode=True,
@@ -533,22 +507,19 @@ def test_properties(image_files: Path):
 
     # test a flat image (RGB PNG)
     with iio.v3.imopen(image_files / "chelsea.png", "r", plugin="pillow") as file:
-        properties = file.properties(index=0)
+        properties = file.properties()
 
     assert properties.shape == (300, 451, 3)
     assert properties.dtype == np.uint8
 
     # test a ndimage (GIF)
-    properties = iio.v3.improps(
-        image_files / "newtonscradle.gif", plugin="pillow", index=None
-    )
+    properties = iio.v3.improps(image_files / "newtonscradle.gif", plugin="pillow")
     assert properties.shape == (36, 150, 200, 3)
     assert properties.dtype == np.uint8
     assert properties.is_batch is True
 
     # test a flat gray image
-    with iio.v3.imopen(image_files / "text.png", "r", plugin="pillow") as file:
-        properties = file.properties(index=0)
+    properties = iio.v3.improps(image_files / "text.png", plugin="pillow", index=0)
 
     assert properties.shape == (172, 448)
     assert properties.dtype == np.uint8
@@ -568,9 +539,13 @@ def test_metadata(test_images):
 
 def test_apng_reading(tmp_path, test_images):
     # create a APNG
-    img = iio.v3.imread(test_images / "newtonscradle.gif", index=None)
+    img = iio.v3.imread(test_images / "newtonscradle.gif")
     iio.v3.imwrite(tmp_path / "test.apng", img)
 
+    props = iio.v3.improps(tmp_path / "test.apng")
+    assert props.shape == (36, 150, 200, 3)
+    assert props.is_batch is True
+
     # test single image read
     with Image.open(tmp_path / "test.apng") as im:
         im.seek(8)
@@ -579,7 +554,7 @@ def test_apng_reading(tmp_path, test_images):
     assert np.allclose(actual, expected)
 
     # test reading all frames
-    all_frames = iio.v3.imread(tmp_path / "test.apng", index=None)
+    all_frames = iio.v3.imread(tmp_path / "test.apng")
     with Image.open(tmp_path / "test.apng") as im:
         for idx, frame in enumerate(ImageSequence.Iterator(im)):
             expected = np.asarray(frame)
@@ -587,6 +562,20 @@ def test_apng_reading(tmp_path, test_images):
             assert np.allclose(actual, expected)
 
 
+def test_apng_metadata(tmp_path, test_images):
+    # create a APNG
+    img = iio.v3.imread(test_images / "newtonscradle.gif")
+    iio.v3.imwrite(tmp_path / "test.apng", img)
+
+    metadata = iio.v3.immeta(tmp_path / "test.apng")
+    assert metadata["shape"] == (200, 150)
+    assert metadata["loop"] == 0
+
+    # set default index explicitly
+    metadata2 = iio.v3.immeta(tmp_path / "test.apng", index=...)
+    assert metadata == metadata2
+
+
 def test_write_format_warning():
     frames = iio.v3.imread("imageio:chelsea.png")
     bytes_image = iio.v3.imwrite("<bytes>", frames, format_hint=".png", plugin="pillow")
