@@ -61,9 +61,7 @@ def test_metadata(test_images: Path):
 
 
 def test_properties(test_images: Path):
-    with iio.imopen(
-        str(test_images / "cockatoo.mp4"), "r", plugin="pyav", legacy_mode=False
-    ) as plugin:
+    with iio.imopen(str(test_images / "cockatoo.mp4"), "r", plugin="pyav") as plugin:
         with pytest.raises(IOError):
             # subsampled format
             plugin.properties(format="yuv420p")
@@ -274,7 +272,7 @@ def test_variable_fps_seek(test_images):
     assert np.allclose(actual, expected)
 
 
-def test_multiple_writes(test_images, tmp_path):
+def test_multiple_writes(test_images):
     out_buffer = io.BytesIO()
     with iio.imopen(out_buffer, "w", plugin="pyav", format_hint=".mp4") as file:
         for frame in iio.imiter(
@@ -317,3 +315,36 @@ def test_invalid_resource(tmp_path):
 
     with pytest.raises(IOError):
         iio.imwrite(tmp_path / "foo.abc", img, plugin="pyav")
+
+
+def test_bayer_write():
+    image_shape = (128, 128)
+    image = np.zeros(image_shape, dtype="uint8")
+    buffer = io.BytesIO()
+
+    with iio.imopen(buffer, "w", plugin="pyav", format_hint=".mp4") as file:
+        image[...] = 0
+        for i in range(256):
+            image[::2, ::2] = i
+            file.write(
+                image, is_batch=False, codec="h264", in_pixel_format="bayer_rggb8"
+            )
+
+        image[...] = 0
+        for i in range(256):
+            image[0::2, 1::2] = i
+            image[1::2, 0::2] = i
+            file.write(
+                image, is_batch=False, codec="h264", in_pixel_format="bayer_rggb8"
+            )
+
+        image[...] = 0
+        for i in range(256):
+            image[1::2, 1::2] = i
+            file.write(
+                image, is_batch=False, codec="h264", in_pixel_format="bayer_rggb8"
+            )
+
+    buffer.seek(0)
+    img = iio.imread(buffer, plugin="pyav")
+    assert img.shape == (768, 128, 128, 3)
